@@ -18,7 +18,7 @@ class StdList(c变量):
     def __setitem__(self, key, value):
         return f'{self}[{key}] = {value};'
 
-    def 初始化(self, 初始值):
+    def 初始化(self, 初始值, 强转类型: str | None):
         if self.初始化列表.类型 == cpp类型.ANY:
             return super().初始化(初始值)
         return f'{self.类型} {self.名字}[] = {初始值};'
