@@ -1,5 +1,4 @@
-from .c基础处理 import py类型转c类型, dict初始化列表, c变量, cpp类型
-from typing import Tuple, Union, List
+from .c基础处理 import dict初始化列表, c变量
 from .通用 import toCString
 
 
