@@ -6,7 +6,7 @@ from typing import Callable, Any, List, Dict
 from .c基础处理 import *
 from .StdList import StdList
 from .StdMap import StdUnorderedMap
-from .通用 import 通用信息, 有非英文变量字符, 尝试创建文件夹
+from .通用 import 通用信息, 有非英文变量字符, 尝试创建文件夹, c类型映射
 from .编译 import cpp编译器
 
 
@@ -82,9 +82,10 @@ class c布尔:
 
 
 class py2cpp编译器(ast.NodeVisitor):
-    def __init__(self, 被编译的函数: Callable, c编译器) -> None:
+    def __init__(self, 被编译的函数: Callable, c编译器, 编译为可执行文件文件名: str | None = None) -> None:
         self.被编译的函数 = 被编译的函数
         self.c编译器: cpp编译器 = c编译器
+        self.编译为可执行文件文件名 = 编译为可执行文件文件名
         self.源代码 = inspect.getsource(被编译的函数)
         self.代码哈希值 = hashlib.blake2s(
             self.源代码.encode(), digest_size=8).hexdigest()
@@ -103,33 +104,42 @@ class py2cpp编译器(ast.NodeVisitor):
         self.花括号 = 花括号(self)
         self.正在调用直接函数 = False
         self.正在构建参数 = False
-        self.include目录 = set(['<cstdint>', '<string>'])
+        self.include列表 = set(['<cstdint>', '<string>'])
         self.链接库列表 = set()
         self.库目录 = set()
 
         # 保存函数名
         self.函数名 = 被编译的函数.__name__
-        if 有非英文变量字符(self.函数名):
-            self.c函数名 = f'function_{self.函数名.encode().hex()}'
+        if 编译为可执行文件文件名:
+            self.c函数名 = 'main'
         else:
-            self.c函数名 = self.函数名
-        self.文件前缀 = f'{self.函数名}_@'
+            if 有非英文变量字符(self.函数名):
+                self.c函数名 = f'function_{self.函数名.encode().hex()}'
+            else:
+                self.c函数名 = self.函数名
+        代码文件地址 = inspect.getfile(被编译的函数)
+        文件名 = os.path.split(代码文件地址)[1]
+        文件名hash = hashlib.blake2s(代码文件地址.encode(), digest_size=8).hexdigest()
+        self.文件前缀 = f'{文件名hash}_{文件名}_{self.函数名}_@'
 
     def 编译(self):
         语法树 = ast.parse(self.源代码)
         self.visit(语法树)
         # 读取依赖函数
-        cpps = [f'{通用信息.工作文件夹地址}/{self.获取cpp文件名()}']
+        cpps = set([f'{通用信息.工作文件夹地址}/{self.获取cpp文件名()}'])
         for b in self.依赖函数:
-            self.include目录.add(f'"{b.获取h文件名()}"')
-            cpps.append(f'{通用信息.工作文件夹地址}/{b.获取cpp文件名()}')
+            self.include列表.add(f'"{b.获取h文件名()}"')
+            cpps.add(f'{通用信息.工作文件夹地址}/{b.获取cpp文件名()}')
 
         self.将代码保存到文件()
-        self.c编译器.include目录列表 = list(self.include目录)
         self.c编译器.库目录列表 = list(self.库目录)
         self.c编译器.链接库 = list(self.链接库列表)
         输出路径 = f'{通用信息.工作文件夹地址}/{self.获取库文件名()}'
-        self.c编译器.编译为动态库(cpps, 输出路径=输出路径)
+        if self.编译为可执行文件文件名:
+            self.c编译器.添加编译选项("-O2")
+            self.c编译器.编译文件(list(cpps), 输出路径=输出路径)
+        else:
+            self.c编译器.编译为动态库(list(cpps), 输出路径=输出路径)
 
     def 添加c代码(self, 代码: str):
         self.代码序列.append(c代码(代码, self.当前上下文层级))
@@ -189,7 +199,7 @@ class py2cpp编译器(ast.NodeVisitor):
 
         if isinstance(value, ast.Attribute):
             对象 = self.获取值(value.value)
-            if isinstance(对象, c变量):
+            if isinstance(对象, (c变量, c类型映射)):
                 return c获取属性(对象, value.attr)
             if 对象 is None:
                 self.抛出代码异常(f'没找到{value.value}', value)
@@ -198,25 +208,25 @@ class py2cpp编译器(ast.NodeVisitor):
         if isinstance(value, ast.UnaryOp):
             operand = self.获取值(value.operand)
             if isinstance(value.op, ast.UAdd):
-                return f'+{operand}'
+                return f'(+{operand})'
             if isinstance(value.op, ast.USub):
-                return f'-{operand}'
+                return f'(-{operand})'
             if isinstance(value.op, ast.Not):
-                return f'!{operand}'
+                return f'(!{operand})'
             if isinstance(value.op, ast.Invert):
-                return f'~{operand}'
+                return f'(~{operand})'
 
         if isinstance(value, ast.BoolOp):
             if isinstance(value.op, ast.And):
-                return '&&'.join([str(self.获取值(v)) for v in value.values])
+                return '&&'.join([f'({self.获取值(v)})' for v in value.values])
             if isinstance(value.op, ast.Or):
-                return '||'.join([str(self.获取值(v)) for v in value.values])
+                return '||'.join([f'({self.获取值(v)})' for v in value.values])
 
         if isinstance(value, ast.IfExp):
             test = self.获取值(value.test)
             body = self.获取值(value.body)
             orelse = self.获取值(value.orelse)
-            return f'({test} ? ({body}) : ({orelse}))'
+            return f'(({test}) ? ({body}) : ({orelse}))'
 
         if isinstance(value, ast.Compare):
             return self.计算Compare(value)
@@ -251,19 +261,22 @@ class py2cpp编译器(ast.NodeVisitor):
             left = left if 输出left else ''
             right = self.获取值(comp)
             if isinstance(op, ast.Eq):
-                ret += f'{left} == {right}'
+                ret += f'({left} == {right})'
             if isinstance(op, ast.NotEq):
-                ret += f'{left} != {right}'
+                ret += f'({left} != {right})'
             if isinstance(op, ast.Lt):
-                ret += f'{left} < {right})'
+                ret += f'({left} < {right})'
             if isinstance(op, ast.LtE):
-                ret += f'{left} <= {right}'
+                ret += f'({left} <= {right})'
             if isinstance(op, ast.Gt):
-                ret += f'{left} > {right}'
+                ret += f'({left} > {right})'
             if isinstance(op, ast.GtE):
-                ret += f'{left} >= {right}'
+                ret += f'({left} >= {right})'
             left = right
             输出left = False
+        # 如果只有一项, 不需要多层括号
+        if len(node.ops) == 1:
+            return ret[1:]
         return ret + ')'
 
     def 计算二元运算(self, node: ast.BinOp | ast.AugAssign):
@@ -275,43 +288,61 @@ class py2cpp编译器(ast.NodeVisitor):
             right = self.获取值(node.value)
         op = node.op
         if isinstance(op, ast.Add):
-            return f'{left} + {right}'
+            return f'({left} + {right})'
         if isinstance(op, ast.Sub):
-            return f'{left} - {right}'
+            return f'({left} - {right})'
         if isinstance(op, ast.Mult):
-            return f'{left} * {right}'
+            return f'({left} * {right})'
         if isinstance(op, (ast.Div, ast.FloorDiv)):
-            return f'{left} / {right}'
+            return f'({left} / {right})'
         if isinstance(op, ast.Mod):
-            return f'{left} % {right}'
-        if isinstance(op, ast.Pow):
-            return f'{left} ** {right}'
+            return f'({left} % {right})'
         if isinstance(op, ast.BitAnd):
-            return f'{left} & {right}'
+            return f'({left} & {right})'
         if isinstance(op, ast.BitOr):
-            return f'{left} | {right}'
+            return f'({left} | {right})'
         if isinstance(op, ast.BitXor):
-            return f'{left} ^ {right}'
+            return f'({left} ^ {right})'
         if isinstance(op, ast.LShift):
-            return f'{left} << {right}'
+            return f'({left} << {right})'
         if isinstance(op, ast.RShift):
-            return f'{left} >> {right}'
+            return f'({left} >> {right})'
 
         self.抛出代码异常(f"暂不支持的运算符: {type(op).__name__}", node)
 
+    def 构建参数列表文本(self, args: List[ast.expr]):
+        参数列表 = [str(self.获取值(arg)) for arg in args]
+        return ','.join(参数列表)
+
     def 调用Call(self, node: ast.Call):
-        # 构建函数
+        # 获取函数
         fn = self.获取值(node.func)
+
+        # 如果是创建类型
+        if inspect.isclass(fn):
+            c类型 = 通用信息.类型映射表.get(fn)
+            if c类型 is not None:
+                for include目录 in c类型.include目录:
+                    self.include列表.add(include目录)
+                for 链接库 in c类型.库列表:
+                    self.链接库列表.add(链接库)
+                for 库目录 in c类型.库目录:
+                    self.库目录.add(库目录)
+                参数文本 = self.构建参数列表文本(node.args)
+                return c函数调用(c类型, 参数文本)
+
+        # 直接调用函数
         if fn in 通用信息.直接调用函数:
             参数列表 = [self.获取值(arg) for arg in node.args]
             self.正在调用直接函数 = True
             return fn(*参数列表)
 
+        # python函数对应的c++函数
         映射函数 = 通用信息.函数映射表.get(fn)
         if 映射函数 is not None:
             参数列表 = [self.获取值(arg) for arg in node.args]
             for include目录 in 映射函数.include目录:
-                self.include目录.add(include目录)
+                self.include列表.add(include目录)
             for 链接库 in 映射函数.库列表:
                 self.链接库列表.add(链接库)
             for 库目录 in 映射函数.库目录:
@@ -319,23 +350,25 @@ class py2cpp编译器(ast.NodeVisitor):
 
             return 映射函数.目标函数(*参数列表)
 
-        if not isinstance(fn, Callable):
+        if not isinstance(fn, Callable) and not isinstance(fn, c获取属性):
             self.抛出代码异常(f'{ast.dump(node.func)} 没找到', node)
 
         if len(node.keywords) > 0:
-            self.代码异常抛出器('暂不支持 keywords 函数调用', node)
-
-        参数列表 = [str(self.获取值(arg)) for arg in node.args]
-        参数字符串 = ','.join(参数列表)
+            self.抛出代码异常('暂不支持 keywords 函数调用', node)
 
+        参数文本 = self.构建参数列表文本(node.args)
+        # 是否是编译好的函数
         if isinstance(fn, py2cpp编译器):
             self.依赖函数.append(fn)
-            return c函数调用(fn.c函数名, 参数字符串)
+            return c函数调用(fn.c函数名, 参数文本)
+        elif isinstance(fn, c获取属性):
+            return c函数调用(fn, 参数文本)
+        # 编译该函数
         else:
             依赖函数编译器 = self.__class__(fn, self.c编译器)
             依赖函数编译器.编译()
             self.依赖函数.append(依赖函数编译器)
-            return c函数调用(fn.__name__, 参数字符串)
+            return c函数调用(fn.__name__, 参数文本)
 
     def 获取Subscript(self, node: ast.Subscript):
         对象 = self.获取值(node.value)
@@ -352,18 +385,18 @@ class py2cpp编译器(ast.NodeVisitor):
         return c获取索引项目(对象, 切片)
 
     def visit_FunctionDef(self, node: ast.FunctionDef) -> Any:
-        self.visit_核函数所有参数(node.args)
-
         with self.花括号:
+            self.visit_核函数所有参数(node.args)
             self.visit_核函数体(node.body)
 
         # 读取返回类型
         if isinstance(node.returns, ast.Name):
             返回py类型 = self.获取值(node.returns)
             self.返回类型 = py类型转c类型(返回py类型)
-            self.ctypes返回类型 = py类型转ctypes类型(返回py类型)
-            if self.ctypes返回类型 is None:
-                self.抛出代码异常(f'不支持的返回值类型{返回py类型}', node)
+            if not self.编译为可执行文件文件名:
+                self.ctypes返回类型 = py类型转ctypes类型(返回py类型)
+                if self.ctypes返回类型 is None:
+                    self.抛出代码异常(f'不支持的返回值类型{返回py类型}', node)
         elif isinstance(node.returns, (tuple)):
             self.抛出代码异常('函数只能有一个返回值', node)
         else:
@@ -394,10 +427,11 @@ class py2cpp编译器(ast.NodeVisitor):
             self.抛出代码异常(f'不支持的类型{类型}', node)
         参数变量 = c变量(str(c类型), 参数名, True, 参数值)
         self.参数变量[参数名] = 参数变量
-        ctypes类型 = py类型转ctypes类型(类型)
-        if ctypes类型 is None:
-            self.抛出代码异常(f'不支持的参数类型{类型}', node)
-        self.ctypes类型.append(ctypes类型)
+        if not self.编译为可执行文件文件名:
+            ctypes类型 = py类型转ctypes类型(类型)
+            if ctypes类型 is None:
+                self.抛出代码异常(f'不支持的参数类型{类型}', node)
+            self.ctypes类型.append(ctypes类型)
         前置处理函数 = self.参数不同c类型前置处理函数.get(类型)
         if 前置处理函数 is not None:
             前置处理函数(self, 参数变量, node)
@@ -420,6 +454,7 @@ class py2cpp编译器(ast.NodeVisitor):
         else:
             self.添加c代码(';')
         if len(node.orelse) > 0:
+            self.添加c代码('else')
             with self.花括号:
                 for stmt in node.orelse:
                     self.visit(stmt)
@@ -444,11 +479,11 @@ class py2cpp编译器(ast.NodeVisitor):
             if fn is range:
                 参数 = [self.获取值(arg) for arg in node.iter.args]
                 if len(参数) == 1:
-                    代码 = f'for (int64_t {目标} = 0; i < {参数[0]}; ++i)'
+                    代码 = f'for (int64_t {目标} = 0; {目标} < {参数[0]}; ++{目标})'
                 elif len(参数) == 2:
-                    代码 = f'for (int64_t {目标} = {参数[0]}; i < {参数[1]}; ++i)'
+                    代码 = f'for (int64_t {目标} = {参数[0]}; {目标} < {参数[1]}; ++{目标})'
                 elif len(参数) == 3:
-                    代码 = f'for (int64_t {目标} = {参数[0]}; i < {参数[1]}; i += {参数[2]})'
+                    代码 = f'for (int64_t {目标} = {参数[0]}; {目标} < {参数[1]}; {目标} += {参数[2]})'
             else:
                 调用代码 = self.调用Call(node.iter)
                 代码 = f'for (auto {目标} : {调用代码})'
@@ -489,9 +524,9 @@ class py2cpp编译器(ast.NodeVisitor):
 
     def visit_Call(self, node: ast.Call) -> Any:
         代码 = self.调用Call(node)
-        self.添加c代码(str(代码))
+        self.添加c代码(f'{代码};')
 
-    def _赋值(self, target, 值, node):
+    def _赋值(self, target, 值, node, c强转类型: str | None = None):
         目标 = self.获取值(target)
         if self.正在调用直接函数:
             if isinstance(target, ast.Name):
@@ -505,19 +540,22 @@ class py2cpp编译器(ast.NodeVisitor):
             if isinstance(target, ast.Name):
                 if isinstance(值, dict初始化列表):
                     目标 = StdUnorderedMap(值, target.id, False)
-                    self.include目录.add('<unordered_map>')
+                    self.include列表.add('<unordered_map>')
                 elif isinstance(值, list初始化列表):
                     目标 = StdList(值, target.id, False)
                     if 值.类型 == cpp类型.ANY:
-                        self.include目录.add('<any>')
+                        self.include列表.add('<any>')
                 else:
                     目标 = c变量('auto', target.id, False)
-                self.添加c代码(目标.初始化(值))
+                self.添加c代码(目标.初始化(值, c强转类型))
                 self.添加c变量(目标)
             else:
                 self.抛出代码异常('for中target必须是ast.Name', node)
         else:
-            self.添加c代码(f'{目标} = {值};')
+            if c强转类型:
+                self.添加c代码(f'{目标} = {c强转类型}({值});')
+            else:
+                self.添加c代码(f'{目标} = {值};')
 
     def visit_Assign(self, node: ast.Assign) -> Any:
         值 = self.获取值(node.value)
@@ -528,6 +566,16 @@ class py2cpp编译器(ast.NodeVisitor):
         值 = self.计算二元运算(node)
         self._赋值(node.target, 值, node)
 
+    def visit_AnnAssign(self, node: ast.AnnAssign) -> Any:
+        值 = self.获取值(node.value)
+        目标类型 = self.获取值(node.annotation)
+        if 目标类型 is None:
+            self.抛出代码异常(f'不支持的类型:{node.annotation}', node)
+        c类型 = py类型转c类型(目标类型)
+        if c类型 is None:
+            self.抛出代码异常(f'不支持的类型:{目标类型}', node)
+        self._赋值(node.target, 值, node, str(c类型))
+
     def 获取定义(self):
         参数定义 = []
         for arg in self.参数变量.values():
@@ -539,11 +587,11 @@ class py2cpp编译器(ast.NodeVisitor):
         return f'extern "C" {self.返回类型} /*{self.函数名}*/ {self.c函数名} ({参数定义文本})'
 
     def 获取includes(self):
-        include代码列表 = [f'#include {目录}' for 目录 in sorted(self.include目录)]
+        include代码列表 = [f'#include {目录}' for 目录 in sorted(self.include列表)]
         return '\n'.join(include代码列表)
 
     def 获取h代码(self):
-        return self.获取includes() + '\n' + self.获取定义() + ';'
+        return '#pragma once\n' + self.获取includes() + '\n' + self.获取定义() + ';'
 
     def 获取无后缀文件名(self):
         return f'{self.文件前缀}{self.代码哈希值}'
@@ -555,6 +603,8 @@ class py2cpp编译器(ast.NodeVisitor):
         return f'{self.获取无后缀文件名()}.cpp'
 
     def 获取库文件名(self):
+        if self.编译为可执行文件文件名:
+            return self.编译为可执行文件文件名
         return f'{self.获取无后缀文件名()}.so'
 
     def 获取cpp代码(self):
@@ -569,6 +619,10 @@ class py2cpp编译器(ast.NodeVisitor):
             if 文件名.startswith(self.文件前缀):
                 文件路径 = os.path.join(通用信息.工作文件夹地址, 文件名)
                 os.remove(文件路径)
+        if self.编译为可执行文件文件名:
+            文件路径 = os.path.join(通用信息.工作文件夹地址, self.编译为可执行文件文件名)
+            if os.path.exists(文件路径):
+                os.remove(文件路径)
 
     def 将代码保存到文件(self):
         self.根据缀删除文件()
@@ -580,6 +634,8 @@ class py2cpp编译器(ast.NodeVisitor):
             fp.write(cpp代码)
 
     def 加载库(self):
+        if self.编译为可执行文件文件名:
+            return
         库地址 = f'{通用信息.工作文件夹地址}/{self.获取库文件名()}'
         self.目标库 = ctypes.CDLL(库地址)
         self.cpp函数 = self.目标库[self.c函数名]
@@ -587,15 +643,19 @@ class py2cpp编译器(ast.NodeVisitor):
         self.cpp函数.restype = self.ctypes返回类型
 
     def __call__(self, *args: Any, **kwds: Any) -> Any:
-        return self.cpp函数(*args)
+        if self.编译为可执行文件文件名:
+            库文件名 = self.获取库文件名()
+            可执行文件地址 = f'{通用信息.工作文件夹地址}/{库文件名}'
+            raise Exception(f"请直接在命令行执行[{可执行文件地址}]")
+        else:
+            return self.cpp函数(*args)
 
 
-def jit(jit编译器类=None, cpp编译器类=None, 每次运行都重新编译: bool = False):
+def jit(jit编译器类=None, cpp编译器类=None, 每次运行都重新编译: bool = False, 编译为可执行文件文件名: str | None = None):
     def 编译函数(fn: Callable):
-        # print(ast.dump(语法树, indent=' '))
         _c编译器类 = cpp编译器类 or cpp编译器
         _jit编译器类 = jit编译器类 or py2cpp编译器
-        c语言函数编译器 = _jit编译器类(fn, _c编译器类())
+        c语言函数编译器 = _jit编译器类(fn, _c编译器类(), 编译为可执行文件文件名)
         库文件名 = c语言函数编译器.获取库文件名()
         if 每次运行都重新编译 or not os.path.exists(f'{通用信息.工作文件夹地址}/{库文件名}'):
             c语言函数编译器.编译()
