@@ -9,6 +9,7 @@ class cpp编译器:
         self.库目录列表 = []
         self.链接库 = []
         self.编译选项 = []
+        self.输出编译指令 = False
 
     def 设置编译器(self, 编译器地址: str):
         self.编译器 = 编译器地址
@@ -49,6 +50,8 @@ class cpp编译器:
             编译指令.append(文件路径)
         编译指令.append('-o')
         编译指令.append(输出路径)
+        if self.输出编译指令:
+            print(编译指令)
         subprocess.run(编译指令)
 
     def 编译为动态库(self, 文件路径: Union[List[str], str], 输出路径: str):
